/* --------------- MD2 computation ---------------------------------------- */



/* ------- start of RSA copyrighted code --------- */

/* some changes were made by Joachim Schurig 28/04/96 to encapsulate
   the code in PASCAL */

/*  Copyright (C) 1990-2, RSA Data Security, Inc. Created 1990. All
    rights reserved.

    License to copy and use this software is granted for
    non-commercial Internet Privacy-Enhanced Mail provided that it is
    identified as the "RSA Data Security, Inc. MD2 Message Digest
    Algorithm" in all material mentioning or referencing this software
    or this function.

    RSA Data Security, Inc. makes no representations concerning either
    the merchantability of this software or the suitability of this
    software for any particular purpose. It is provided "as is"
    without express or implied warranty of any kind.

    These notices must be retained in any copies of any part of this
    documentation and/or software.
  */


/* by DG6VJ to use the File standalone */
typedef unsigned char uchar;
/* Boolean */
typedef unsigned char boolean;
#define true 1
#define false 0

#define lower Klein


typedef uchar MD2barr16[16];

typedef struct MD2_CTX {
  MD2barr16 state, checksum;
  long count;
  MD2barr16 buffer;
} MD2_CTX;


/*
 *  Permutation of 0..255 constructed from the digits of pi. It gives a
 *  "random" nonlinear byte substitution operation.
 */


typedef uchar PI_SUBSTTYPE[256];


typedef MD2barr16 PADDINGTYPE[17];


static PI_SUBSTTYPE PI_SUBST = {
  41, 46, 67, 201, 162, 216, 124, 1, 61, 54, 84, 161, 236, 240, 6, 19, 98,
  167, 5, 243, 192, 199, 115, 140, 152, 147, 43, 217, 188, 76, 130, 202, 30,
  155, 87, 60, 253, 212, 224, 22, 103, 66, 111, 24, 138, 23, 229, 18, 190, 78,
  196, 214, 218, 158, 222, 73, 160, 251, 245, 142, 187, 47, 238, 122, 169,
  104, 121, 145, 21, 178, 7, 63, 148, 194, 16, 137, 11, 34, 95, 33, 128, 127,
  93, 154, 90, 144, 50, 39, 53, 62, 204, 231, 191, 247, 151, 3, 255, 25, 48,
  179, 72, 165, 181, 209, 215, 94, 146, 42, 172, 86, 170, 198, 79, 184, 56,
  210, 150, 164, 125, 182, 118, 252, 107, 226, 156, 116, 4, 241, 69, 157, 112,
  89, 100, 113, 135, 32, 134, 91, 207, 101, 230, 45, 168, 2, 27, 96, 37, 173,
  174, 176, 185, 246, 28, 70, 97, 105, 52, 64, 126, 15, 85, 71, 163, 35, 221,
  81, 175, 58, 195, 92, 249, 206, 186, 197, 234, 38, 44, 83, 13, 110, 133, 40,
  132, 9, 211, 223, 205, 244, 65, 129, 77, 82, 106, 220, 55, 200, 108, 193,
  171, 250, 36, 225, 123, 8, 12, 189, 177, 74, 120, 136, 149, 139, 227, 99,
  232, 109, 233, 203, 213, 254, 59, 0, 29, 57, 242, 239, 183, 14, 102, 88,
  208, 228, 166, 119, 114, 248, 235, 117, 75, 10, 49, 68, 80, 180, 143, 237,
  31, 26, 219, 153, 141, 51, 159, 17, 131, 20
};

static PADDINGTYPE PADDING;
static boolean PADDINGOK;
/* must be initialized with FALSE at program startup (Joachim Schurig) */

/*
 *  initializes the padding tab, cannot be assigned directly in PASCAL code
 *  added by Joachim Schurig
 */

static void init_padding(void)
{
  short x, y;

  PADDING[0][0] = 0;

  for (x = 1; x <= 16; x++) {
    for (y = 0; y < x; y++)
      PADDING[x][y] = x;
  }

  PADDINGOK = true;
}

/* MD2 initialization. Begins an MD2 operation, writing a new context. */

static void MD2Init(MD2_CTX *context)
{
  if (!PADDINGOK)   /* added for PASCAL */
    init_padding();
  context->count = 0;
  memset(context->state, 0, sizeof(MD2barr16));
  memset(context->checksum, 0, sizeof(MD2barr16));
}

/* MD2 basic transformation. Transforms state and updates checksum based on block. */

static void MD2Transform(uchar *state, uchar *checksum, uchar *block)
{
  long i, j, t;
  uchar x[48];

  /* Form encryption block from state, block, state ^ block. */
  memcpy(x, state, 16);
  memcpy(&x[16], block, 16);
  for (i = 0; i <= 15; i++)
    x[i + 32] = state[i] ^ block[i];

  /* Encrypt block (18 rounds). */
  t = 0;
  for (i = 0; i <= 17; i++) {
    for (j = 0; j <= 47; j++) {
      t = x[j] ^ PI_SUBST[t];
      x[j] = t;
    }
    t = (t + i) & 0xff;
  }

  /* Save new state */
  memcpy(state, x, 16);

  /* Update checksum. */
  t = checksum[15];
  for (i = 0; i <= 15; i++) {
    t = checksum[i] ^ PI_SUBST[block[i] ^ t];
    checksum[i] = t;
  }

  /* Zeroize sensitive information. */
  memset(x, 0, sizeof(uchar) * 48);
}

/*
 *  MD2 block update operation. Continues an MD2 message-digest
 *  operation, processing another message block, and updating the
 *  context.
 */

static void MD2Update(MD2_CTX *context, uchar *input, long inputLen)
{
  long i, index, partLen;

  /* Update number of bytes mod 16 */
  index = context->count;
  context->count = (index + inputLen) & 0xf;

  partLen = 16 - index;

  /* Transform as many times as possible. */
  if (inputLen >= partLen) {
    memcpy(&context->buffer[index], input, partLen);
    MD2Transform(context->state, context->checksum, context->buffer);

    i = partLen;
    while (i + 15 < inputLen) {
      MD2Transform(context->state, context->checksum, (uchar *)(&input[i]));
      i += 16;
    }

    index = 0;
  } else
    i = 0;

  /* Buffer remaining input */
  memcpy(&context->buffer[index], (uchar *)(&input[i]), inputLen - i);
}

/*
 *  MD2 finalization. Ends an MD2 message-digest operation, writing the
 *  message digest and zeroizing the context.
 */

static void MD2Final(uchar *digest, MD2_CTX *context)
{
  long index, padLen;

  /* Pad out to multiple of 16 */
  index = context->count;
  padLen = 16 - index;
  MD2Update(context, PADDING[padLen], padLen);

  /* Extend with checksum */
  MD2Update(context, context->checksum, 16);

  /* Store state in digest */
  memcpy(digest, context->state, 16);

  /* Zeroize sensitive information */
  memset((uchar *)context, 0, sizeof(MD2_CTX));
}

/* ------- end of RSA copyrighted code ------- */



void calc_MD2_pw(char *MD2prompt, char *MD2pw, char *MD2result)
{
  MD2_CTX context;
  MD2barr16 digest;
  short i, n, len;
  long bc;
  char c1, c2;
  uchar *buffp;
  char buff[256];
  short FORLIM;

  buffp = buff;
  bc = 0;
  FORLIM = strlen(MD2prompt);
  for (i = 0; i < FORLIM; i++) {
    buffp[bc] = MD2prompt[i];
    bc++;
  }
  FORLIM = strlen(MD2pw);
  for (i = 0; i < FORLIM; i++) {
    buffp[bc] = MD2pw[i];
    bc++;
  }

  buffp[bc] = 0;

  MD2Init(&context);
  len = bc;

  i = 0;

  while (i < len) {
    if (len - i > 16)
      n = 16;
    else
      n = len - i;
    MD2Update(&context, (uchar *)(&buffp[i]), n);
    i += 16;
  }
  MD2Final(digest, &context);

  MD2result[0] = '\0';
  for (i = 0; i <= 15; i++) {
    int2hchar(digest[i], &c1, &c2);
    sprintf(MD2result + strlen(MD2result), "%c%c", c1, c2);
  }
  lower(MD2result);
}


void calc_MD2_prompt(char *MD2prompt)
{
  short x;
  char c;

  MD2prompt[0] = '\0';
  for (x = 1; x <= 10; x++) {
    do {
      c = dp_randomize(48, 122);
    } while (!isalnum(c));
    sprintf(MD2prompt + strlen(MD2prompt), "%c", c);
  }
}


/* ------------------------------------------------------------------ */



